def node_function_1(data):
    """
    An example function that processes data and returns a modified version.

    Args:
        data: The input data to be processed.

    Returns:
        str: The processed data.
    """
    print("Input Data:", data)
    processed_data = data + "asasdasdd"
    print("Processed Data:", processed_data)
    return processed_data

def node_function_2(data):
    """
    An example function that returns the input data unchanged.

    Args:
        data: The input data.

    Returns:
        str: The input data.
    """
    print("Input Data:", data)
    return data
